/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.file.storage;

import java.util.ArrayList;
import java.util.Collection;
import com.openexchange.file.storage.Document;
import com.openexchange.file.storage.File;
import com.openexchange.file.storage.File.Field;

/**
 * {@link ContentTypeFileRecognizer} recognizes a file as encrypted based on it's content-type.
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.8.3
 */
public class ContentTypeFileRecognizer implements EncryptedFileRecognizer {

    private static final String PGP_CONTENT_TYPE = "application/pgp-encrypted";
    private static final String ORIGINAL_MIME_TYPE = "OrigMime";

    /* (non-Javadoc)
     * @see com.openexchange.guard.file.storage.EncryptedFileRecognizer#isEncrypted(com.openexchange.file.storage.File)
     */
    @Override
    public boolean isEncrypted(File file) {
        if(file != null) {
            return PGP_CONTENT_TYPE.equals(file.getFileMIMEType());
        }
        return false;
    }

    /* (non-Javadoc)
     * @see com.openexchange.guard.file.storage.EncryptedFileRecognizer#markEncrypted(com.openexchange.file.storage.File)
     */
    @Override
    public Collection<Field> markEncrypted(File file) {
        ArrayList<Field> ret = new ArrayList<Field>();
        if(!isEncrypted(file)) {

            //Saving original content-type into meta-data
            if (!file.getMeta().containsKey(ORIGINAL_MIME_TYPE)) {
                file.getMeta().put(ORIGINAL_MIME_TYPE, file.getFileMIMEType());
                ret.add(Field.META);
            }

            //Setting content-type
            file.setFileMIMEType(PGP_CONTENT_TYPE);
            ret.add(Field.FILE_MIMETYPE);
        }
        return ret;
    }

    /* (non-Javadoc)
     * @see com.openexchange.guard.file.storage.EncryptedFileRecognizer#markEncrypted(com.openexchange.file.storage.Document)
     */
    @Override
    public Collection<Field> markEncrypted(Document document) {
        return markEncrypted(document.getFile());
    }

    /* (non-Javadoc)
     * @see com.openexchange.guard.file.storage.EncryptedFileRecognizer#markDecrypted(com.openexchange.file.storage.File)
     */
    @Override
    public Collection<Field> markDecrypted(File file) {
        ArrayList<Field> ret = new ArrayList<Field>();
        if (file.getMeta().containsKey(ORIGINAL_MIME_TYPE)) {
            //Get the file's original content-type back from the meta-data
            String originalMimeType = (String)file.getMeta().get(ORIGINAL_MIME_TYPE);
            file.getMeta().remove(ORIGINAL_MIME_TYPE);
            ret.add(Field.META);
            //..and restore it
            file.setFileMIMEType(originalMimeType);
            ret.add(Field.FILE_MIMETYPE);
        }
        return ret;
    }

    /* (non-Javadoc)
     * @see com.openexchange.guard.file.storage.EncryptedFileRecognizer#markDecrypted(com.openexchange.file.storage.Document)
     */
    @Override
    public Collection<Field> markDecrypted(Document document) {
        return markDecrypted(document.getFile());
    }

}
