/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.json.actions;

import java.util.Arrays;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.ajax.requesthandler.AJAXRequestData;
import com.openexchange.ajax.requesthandler.AJAXRequestResult;
import com.openexchange.exception.OXException;
import com.openexchange.server.ServiceLookup;
import com.openexchange.session.Session;
import com.openexchange.tools.servlet.AjaxExceptionCodes;
import com.openexchange.tools.session.ServerSession;

/**
 * {@link SessionKeysAction}
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v2.10.0
 */
public class SessionKeysAction extends AbstractGuardAction {

    private static final String JSON_FIELD_FOLDER = "folder";
    private static final String JSON_FIELD_ID = "id";
    private static final String JSON_MODULE = "module";
    private final SessionKeyExtractionHandler[] extractionHandlers;

    /**
     * Initializes a new {@link SessionKeysAction}.
     *
     * @param services
     */
    public SessionKeysAction(ServiceLookup services, SessionKeyExtractionHandler...extractionHandlers) {
        super(services);
        this.extractionHandlers = extractionHandlers;
    }

    private JSONObject asJsonResult(String[] sessions) throws JSONException {
        JSONObject result = new JSONObject();
        if (sessions != null) {
            result.put("encryptedSessionKeyPackets", new JSONArray(Arrays.asList(sessions)));
        }
        return result;
    }

    private SessionKeyExtractionHandler getExtractionHandlerFor(JSONObject json) throws JSONException {
        for (SessionKeyExtractionHandler sessionKeyExtractionHandler : extractionHandlers) {
            if(sessionKeyExtractionHandler.canHandle(json)) {
               return sessionKeyExtractionHandler;
            }
        }
        return null;
    }

    private AJAXRequestResult doPerformPost(AJAXRequestData requestData, Session session) throws OXException, JSONException {
        final JSONObject json = parseStreamToJson(requestData);
        final SessionKeyExtractionHandler extractionHandler = getExtractionHandlerFor(json);
        if(extractionHandler != null) {
            String[] sessions = extractionHandler.handle(json, session);
            requestData.setResponseHeader("content-type", "application/json; charset=UTF-8");
            return new AJAXRequestResult(asJsonResult(sessions), "json");
        }
        else {
            throw AjaxExceptionCodes.JSON_ERROR.create("No suitable extraction handler found.");
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.guard.json.actions.AbstractGuardAction#doPerform(com.openexchange.ajax.requesthandler.AJAXRequestData, com.openexchange.tools.session.ServerSession)
     */
    @Override
    protected AJAXRequestResult doPerform(AJAXRequestData requestData, ServerSession session) throws OXException, JSONException {
        if (requestData.optHttpServletRequest().getMethod().equals("POST")) {
            return doPerformPost(requestData, session);
        } else {
            return new AJAXRequestResult(null, "json");
        }
    }
}
